
lgr = love.graphics


--Holds the offsets to put the origin of the grid in the center of the screen
cX = lgr.getWidth( ) / 2
cY = lgr.getHeight() / 2
radius = 100

tStation = lgr.newImage("textures/station.png")
tCannon = lgr.newImage("textures/cannon.png")
tCity = lgr.newImage("textures/city.png")


function love.load()
	require("entities")
	require("constants")
	ents.Register("rocket")
	ents.Register("city")
	ents.Register("turret")
	-- Register our "Entities" in the game.
	lgr.setBackgroundColor( 0, 0, 0 )
	--love.keyboard.setKeyRepeat(0.01, 0.2)-- Make it so the keys repeat when we hold them down.
	
	print( -5 % 10 )
end



function drawDebug(dt)
	lgr.setColor(255,255,255,255)
	lgr.print( "FPS: " .. love.timer.getFPS( ), 16, 16, 0, 1, 1 )
	lgr.print( "Num Draws: " .. numDraws, 16, 36, 0, 1, 1 )	
	
	lgr.print( "X: " ..getWorldX() , 16, 54, 0, 1, 1 )	
	lgr.print( "Y: " .. getWorldY(), 16, 70, 0, 1, 1 )	

	lgr.print( "Rot: " .. rot, 16, 130, 0, 1, 1 )
end


function drawObjects(dt)

	-- Center Planet
	lgr.setColor( 50, 50, 50, 85 )
	lgr.circle( "fill", 0, 0, radius, 180 )

	rot = rotation(0,0,getWorldX(), getWorldY())--math.atan2(getWorldX(),  getWorldY())
	--Find current direction vector
	local dirx = math.cos( rot - math.pi/2) 
	local diry = math.sin( rot - math.pi/2)

	lgr.setColor( 0, 250, 0, 255 )
	lgr.circle("line", dirx*radius,  diry*radius, 10, 360) -- Mark the center of the screen.
	lgr.line(0,0,dirx* 600, diry * 600)

	for k,v in pairs(ents.objects) do
		v:draw(dt)
	end
end

function love.draw(dt)
	local x = love.mouse.getX() - cX
	local y = love.mouse.getY() - cY

	numDraws = 0
	
	
	
	lgr.push() --Store original
	-- All of my objects required me to do this in each function so why not save some power and do it first
	lgr.translate(cX -x, cY - y)

	-- Draw our game layout
	drawObjects(dt)
	
	-- Revert to standard grid to draw UI
	lgr.pop()
	
	lgr.setColor( 250, 250, 250, 255 )
	lgr.circle("line", love.mouse.getX(),  love.mouse.getY(), 10, 360) -- Mark the center of the screen.
	--Draw UI
	drawDebug(dt)

end

function love.update(dt)
	--Update all of our objects
	for k,v in pairs(ents.objects) do
		v:update(dt)
	end
	
end

function love.focus(bool)
end

function love.keypressed( key, unicode )
	if key == "f5" then--restart
		love.filesystem.load("main.lua")()
		love.load()
	end
end

function love.keyreleased( key, unicode )
	if key == "r" then
		local city = placeObject("city", rotation(0,0,getWorldX(), getWorldY()))
		city:spawn()
	elseif key == "t" then
		local turret = placeObject("turret", rotation(0,0,getWorldX(), getWorldY()))
		turret:spawn()
	end
end


function love.mousepressed( x, y, button )
end


function love.mousereleased( x, y, button )
	
end

function love.quit()
end


--TODO: Make it go all the directions. only goes \. for now. what about ./ ^\ or /^ ? 
function insideBox( px, py, x1, y1, x2, y2 )
	-- I keep forgetting... Y IS FLIPPED!
	if px >= x1 and px <= x2 and py <= y1 and py >= y2 then
		return true
	else
		return false
	end
end

function clamp(value, min, max)
	if value > max then
		return max
	elseif value < min then
		return min
	else
		return value
	end
end

-- 2 mouse functions for getting where the mouse is in the world
function getWorldX()
	return (love.mouse.getX()-cX) * 2
end
function getWorldY()
	return (cY - love.mouse.getY()) * 2
end

function placeObject(object, rotation)
	local x = math.cos( rotation - math.pi/2) 
	local y = math.sin( rotation - math.pi/2)
	local object =  ents.Create(object,x*radius,y*radius)
	object.rot = rotation
	return object
end

function rotation(x1,y1, x2,y2)
	return math.atan2(x2-x1, y2-y1)
end

function distance(x1,y1, x2,y2)
	return math.sqrt( (x2 - x1)^2 + (y2 - y1)^2 )
end
